#!/usr/bin/perl
# -*- project-name: VASM -*-

use strict;
use warnings;
use Test::More qw(no_plan);
use IO::Scalar;

# XML::Writer seems to think it is dealing with a tied handle?
BEGIN { package IO::Scalar; sub BINMODE {} } # UGLY HACK

### use VASM::Catalog::XorgConfig
BEGIN { use_ok('VASM::Catalog::XorgConfig') }

### Ability test
can_ok('VASM::Catalog::XorgConfig', qw/new Store Retrieve Parse Write/);

### new
my $catalog = VASM::Catalog::XorgConfig->new;
# The catalog is a member of the appropriate class?
isa_ok($catalog, 'VASM::Catalog::XorgConfig');

### Store and Retrieve
# Store some things to test later
$catalog->Store(foo => qw/bar baz/);
$catalog->Store(bar => qw/foo baz/);
$catalog->Store(baz => qw/foo bar/);

# Now retrieve some stuff and check it
is_deeply([ $catalog->Retrieve('foo') ], [ qw/bar baz/ ],
          q(Datum at 'foo' contains qw/bar baz/));

### List
# Are the catalog contents valid?
is_deeply([ $catalog->List ], [ qw/foo bar baz/ ],
          'Catalog contents valid');

### Parse
# Instantiate an IO::Scalar containing an XML description of a catalog and
# parse it
my $xmlParseString = q{<?xml version="1.0" encoding="utf-8"?>

<catalog>
  <!-- Generated by VASM -->
  <foo>
    <value>bar</value>
    <value>baz</value>
  </foo>
  <bar>
    <value>foo</value>
    <value>baz</value>
  </bar>
  <baz>
    <value>foo</value>
    <value>bar</value>
  </baz>
</catalog>
};
my $xmlParseHandle = IO::Scalar->new(\$xmlParseString);
my $catalogParse = VASM::Catalog::XorgConfig->new($xmlParseHandle);

# Do $catalogParse and $catalog hold the same values?
is_deeply($catalogParse, $catalog, 'Parsed catalog valid');

### Write
my $xmlWriteString; my $xmlWriteHandle = IO::Scalar->new(\$xmlWriteString);
$catalog->Write($xmlWriteHandle);

# Is the written XML the same as the XML parse string? 
is($xmlWriteString, $xmlParseString, 'Written XML valid');
